@@ -3,6 +3,8 @@ package Config::Any::Perl;
 use strict;
 use warnings;
 
+my %cache;
+
 =head1 NAME
 
 Config::Any::Perl - Load Perl config files
@@ -42,20 +44,33 @@ Attempts to load C<$file> as a Perl file.
 sub load {
     my $class = shift;
     my $file  = shift;
-    return eval { require $file };
+    my $content;
+
+    unless ( $content = $cache{ $file } ) {
+        $content = eval { require $file };
+        $cache{ $file } = $content;
+    }
+
+    return $content;
+}
+
+=head2 is_supported( )
+
+Returns true.
+
+=cut
+
+sub is_supported {
+    return 1;
 }
 
 =head1 AUTHOR
 
-=over 4 
-
-=item * Brian Cassidy E<lt>bricas@cpan.orgE<gt>
-
-=back
+Brian Cassidy E<lt>bricas@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006 by Brian Cassidy
+Copyright 2007 by Brian Cassidy
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
